# -*- coding:utf-8; mode:python3; indent-tabs-mode:nil; c-basic-offset:4; -*-

""" Controller names from their index in the corresponding event type. """

## @namespace midi.controller_names

from . import controller_events

from .controller_events import (
    Sliders7Bits,
    Commands,
    Sliders14Bits,
    Selectors,
    Switches)


# LSB/MSB
# ============================================================================

LSB_SUFFIX = " LSB"
MSB_SUFFIX = " MSB"


# Per controller class
# ============================================================================

# Coarse Sliders
# ----------------------------------------------------------------------------

SLIDER_7_BITS_NAMES = {
    Sliders7Bits.SOUND_VARIATION: "Sound Variation",
    Sliders7Bits.TIMBRE: "Timbre",
    Sliders7Bits.RELEASE_TIME: "Release Time",
    Sliders7Bits.ATTACK_TIME: "Attack Time",
    Sliders7Bits.BRIGHTNESS: "Brightness",
    Sliders7Bits.SOUND_CONTROLLER_6: "Sound Controller 6",
    Sliders7Bits.SOUND_CONTROLLER_7: "Sound Controller 7",
    Sliders7Bits.SOUND_CONTROLLER_8: "Sound Controller 8",
    Sliders7Bits.SOUND_CONTROLLER_9: "Sound Controller 9",
    Sliders7Bits.SOUND_CONTROLLER_10: "Sound Controller 10",
    Sliders7Bits.SOUND_CONTROLLER_11: "Sound Controller 11",
    Sliders7Bits.SOUND_CONTROLLER_12: "Sound Controller 12",
    Sliders7Bits.SOUND_CONTROLLER_13: "Sound Controller 13",
    Sliders7Bits.SOUND_CONTROLLER_14: "Sound Controller 14",
    Sliders7Bits.SOUND_CONTROLLER_15: "Sound Controller 15",
    Sliders7Bits.SOUND_CONTROLLER_16: "Sound Controller 16",
    Sliders7Bits.SOUND_CONTROLLER_17: "Sound Controller 17",
    Sliders7Bits.SOUND_CONTROLLER_18: "Sound Controller 18",
    Sliders7Bits.SOUND_CONTROLLER_19: "Sound Controller 19",
    Sliders7Bits.SOUND_CONTROLLER_20: "Sound Controller 20",
    Sliders7Bits.SOUND_CONTROLLER_21: "Sound Controller 21",
    Sliders7Bits.EFFECTS: "Effects",
    Sliders7Bits.TREMULO: "Tremulo",
    Sliders7Bits.CHORUS: "Chorus",
    Sliders7Bits.DETUNE: "Detune",
    Sliders7Bits.PHASER: "Phaser",
}

UNKNOWN_SLIDER_7_BITS = "Unknown 7 Bits Slider"

assert len(SLIDER_7_BITS_NAMES) <= len(Sliders7Bits)
assert all(x in SLIDER_7_BITS_NAMES for x in Sliders7Bits)


def coarse_slider_name(event):
    """ Coarse slider name for coarse slider index. """
    if not isinstance(event, controller_events.Slider7Bits):
        raise RuntimeError
    index = event.index
    if index in SLIDER_7_BITS_NAMES:
        result = SLIDER_7_BITS_NAMES[index]
    else:
        result = UNKNOWN_SLIDER_7_BITS
    return result


# Commands
# ----------------------------------------------------------------------------

COMMAND_NAMES = {
    Commands.DATA_ENTRY_INC: "Data Entry Increment",
    Commands.DATA_ENTRY_DEC: "Data Entry Decrement",
}

UNKNOWN_COMMAND = "Unknown Command"

assert len(COMMAND_NAMES) <= len(Commands)
assert all(x in COMMAND_NAMES for x in Commands)


def command_name(event):
    """ Switch name for switch index. """
    if not isinstance(event, controller_events.Command):
        raise RuntimeError
    index = event.index
    if index in COMMAND_NAMES:
        result = COMMAND_NAMES[index]
    else:
        result = UNKNOWN_COMMAND
    return result


# Fine Sliders
# ----------------------------------------------------------------------------

SLIDER_14_BITS_NAMES = {
    Sliders14Bits.MODULATION: "Modulation Wheel",
    Sliders14Bits.BREATH: "Breath Control",
    Sliders14Bits.FOOT_CONTROLLER: "Foot Controller",
    Sliders14Bits.PORTAMENTO_TIME: "Portamento Time",
    Sliders14Bits.DATA_ENTRY: "Data Entry",
    Sliders14Bits.CHANNEL_VOLUME: "Channel Volume",
    Sliders14Bits.BALANCE: "Balance",
    Sliders14Bits.PAN: "Pan",
    Sliders14Bits.EXPRESSION: "Expression",
    Sliders14Bits.EFFECT_1: "Effect 1",
    Sliders14Bits.EFFECT_2: "Effect 2",
    Sliders14Bits.GP_CONTROLLER_1: "Multipurpose Controller 1",
    Sliders14Bits.GP_CONTROLLER_2: "Multipurpose Controller 2",
    Sliders14Bits.GP_CONTROLLER_3: "Multipurpose Controller 3",
    Sliders14Bits.GP_CONTROLLER_4: "Multipurpose Controller 4",
}

UNKNOWN_SLIDER_14_BITS = "Unknown 14 Bits Slider"

assert len(SLIDER_14_BITS_NAMES) <= len(Sliders14Bits)
assert all(x in SLIDER_14_BITS_NAMES for x in Sliders14Bits)


def fine_slider_name(event):
    """ Fine slider name for fine slider index. """
    if not isinstance(event, controller_events.Slider14Bits):
        raise RuntimeError
    index = event.index
    msb = event.msb
    if index in SLIDER_14_BITS_NAMES:
        result = SLIDER_14_BITS_NAMES[index]
    else:
        result = UNKNOWN_SLIDER_14_BITS
    if msb:
        result += MSB_SUFFIX
    else:
        result += LSB_SUFFIX
    return result


# Onmi Channels
# ----------------------------------------------------------------------------

OMNI_CHANNELS = "Omni Channels"


# Selectors
# ----------------------------------------------------------------------------

SELECTOR_NAMES = {
    Selectors.BANK: "Bank",
    Selectors.NRPN: "NRPN",
    Selectors.RPN: "RPN",
}

UNKNOWN_SELECTOR = "Unknown Selector"

assert len(SELECTOR_NAMES) <= len(Selectors)
assert all(x in SELECTOR_NAMES for x in Selectors)


def selector_name(event):
    """ Selector name for selector index. """
    if not isinstance(event, controller_events.Selector):
        raise RuntimeError
    index = event.index
    msb = event.msb
    if index in SELECTOR_NAMES:
        result = SELECTOR_NAMES[index]
    else:
        result = UNKNOWN_SELECTOR
    if msb:
        result += MSB_SUFFIX
    else:
        result += LSB_SUFFIX
    return result


# Switches
# ----------------------------------------------------------------------------

SWITCH_NAMES = {
    Switches.SUSTAIN: "Sustain Pedal",
    Switches.PORTAMENTO: "Portamento Pedal",
    Switches.SUSTENUTO: "Sustenuto Pedal",
    Switches.SOFT: "Soft Pedal",
    Switches.LEGATO: "Legato Pedal",
    Switches.HOLD: "Hold Pedal",
}

UNKNOWN_SWITCH = "Unknown Switch"

assert len(SWITCH_NAMES) <= len(Switches)
assert all(x in SWITCH_NAMES for x in Switches)


def switch_name(event):
    """ Switch name for switch index. """
    if not isinstance(event, controller_events.Switch):
        raise RuntimeError
    index = event.index
    if index in SWITCH_NAMES:
        result = SWITCH_NAMES[index]
    else:
        result = UNKNOWN_SWITCH
    return result


# Main
# ============================================================================

def name(event):
    """ Controller name. """

    if isinstance(event, controller_events.Slider7Bits):
        result = coarse_slider_name(event)
    elif isinstance(event, controller_events.Command):
        result = command_name(event)
    elif isinstance(event, controller_events.Slider14Bits):
        result = fine_slider_name(event)
    elif isinstance(event, controller_events.Selector):
        result = selector_name(event)
    elif isinstance(event, controller_events.Switch):
        result = switch_name(event)
    else:
        raise RuntimeError

    return result
