# -*- coding:utf-8; mode:python3; indent-tabs-mode:nil; c-basic-offset:4; -*-

""" GM 1 Instrument, familly and percussion names. """


# GM1 Instrument Names
# ============================================================================

# For use with `midi.ProgramChange` events.
# Note that display number start at 1, while its integer code start at 0.
# See also `GM1_INSTRUMENT_FAMILLY_NAMES`.
# Indexed by zero based program number.
GM1_INSTRUMENT_NAMES = [

    # Piano Familly
    "Acoustic Grand Piano",  # 1
    "Bright Acoustic Piano",  # 2
    "Electric Grand Piano",  # 3
    "Honky‑tonk Piano",  # 4
    "Electric Piano 1",  # 5
    "Electric Piano 2",  # 6
    "Harpsichord",  # 7
    "Clavinet",  # 8

    # Chromatic Percussion Familly
    "Celesta",  # 9
    "Glockenspiel",  # 10
    "Music Box",  # 11
    "Vibraphone",  # 12
    "Marimba",  # 13
    "Xylophone",  # 14
    "Tubular Bells",  # 15
    "Dulcimer",  # 16

    # Organ Familly
    "Drawbar Organ",  # 17
    "Percussive Organ",  # 18
    "Rock Organ",  # 19
    "Church Organ",  # 20
    "Reed Organ",  # 21
    "Accordion",  # 22
    "Harmonica",  # 23
    "Tango Accordion",  # 24

    # Guitar Familly
    "Acoustic Nylon Guitar",  # 25
    "Acoustic Steel Guitar",  # 26
    "Electric Jazz Guitar",  # 27
    "Electric Clean Guitar",  # 28
    "Electric Muted Guitar",  # 29
    "Overdriven Guitar",  # 30
    "Distortion Guitar",  # 31
    "Guitar Harmonics",  # 32

    # Bass Familly
    "Acoustic Bass",  # 33
    "Fingered Electric Bass",  # 34
    "Picked Electric Bass",  # 35
    "Fretless Bass",  # 36
    "Slap Bass 1",  # 37
    "Slap Bass 2",  # 38
    "Synth Bass 1",  # 39
    "Synth Bass 2",  # 40

    # Strings Familly
    "Violin",  # 41
    "Viola",  # 42
    "Cello",  # 43
    "Contrabass",  # 44
    "Tremolo Strings",  # 45
    "Pizzicato Strings",  # 46
    "Orchestral Harp",  # 47
    "Timpani",  # 48

    # Ensemble Familly
    "String Ensemble 1",  # 49
    "String Ensemble 2",  # 50
    "SynthStrings 1",  # 51
    "SynthStrings 2",  # 52
    "Choir Aahs",  # 53
    "Voice Oohs",  # 54
    "Synth Voice",  # 55
    "Orchestra Hit",  # 56

    # Brass Familly
    "Trumpet",  # 57
    "Trombone",  # 58
    "Tuba",  # 59
    "Muted Trumpet",  # 60
    "French Horn",  # 61
    "Brass Section",  # 62
    "SynthBrass 1",  # 63
    "SynthBrass 2",  # 64

    # Reed Familly
    "Soprano Sax",  # 65
    "Alto Sax",  # 66
    "Tenor Sax",  # 67
    "Baritone Sax",  # 68
    "Oboe",  # 69
    "English Horn",  # 70
    "Bassoon",  # 71
    "Clarinet",  # 72

    # Pipe Familly
    "Piccolo",  # 73
    "Flute",  # 74
    "Recorder",  # 75
    "Pan Flute",  # 76
    "Blown Bottle",  # 77
    "Shakuhachi",  # 78
    "Whistle",  # 79
    "Ocarina",  # 80

    # Synth Lead Familly
    "Square Wave (lead 1)",  # 81
    "Sawtooth Wave (lead 2)",  # 82
    "Calliope (lead 3)",  # 83
    "Chiff (lead 4)",  # 84
    "Charang (lead 5)",  # 85
    "Voice (lead 6)",  # 86
    "Fifths (lead 7)",  # 87
    "Bass + Lead (lead 8)",  # 88

    # Synth Pad Familly
    "New Age (pad 1)",  # 89
    "Warm (pad 2)",  # 90
    "Polysynth (pad 3)",  # 91
    "Choir (pad 4)",  # 92
    "Bowed (pad 5)",  # 93
    "Metallic (pad 6)",  # 94
    "Halo (pad 7)",  # 95
    "Sweep (pad 8)",  # 96

    # Synth Effects Familly
    "Rain (fx 1)",  # 97
    "Soundtrack (fx 2)",  # 98
    "Crystal (fx 3)",  # 99
    "Atmosphere (fx 4)",  # 100
    "Brightness (fx 5)",  # 101
    "Goblins (fx 6)",  # 102
    "Echoes (fx 7)",  # 103
    "Sci‑Fi (fx 8)",  # 104

    # Ethnic Familly
    "Sitar",  # 105
    "Banjo",  # 106
    "Shamisen",  # 107
    "Koto",  # 108
    "Kalimba",  # 109
    "Bag Pipe",  # 110
    "Fiddle",  # 111
    "Shanai",  # 112

    # Percussive Familly
    "Tinkle Bell",  # 113
    "Agogo",  # 114
    "Steel Drums",  # 115
    "Woodblock",  # 116
    "Taiko Drum",  # 117
    "Melodic Tom",  # 118
    "Synth Drum",  # 119

    # Sound Effects Familly
    "Reverse Cymbal",  # 120
    "Guitar Fret Noise",  # 121
    "Breath Noise",  # 122
    "Seashore",  # 123
    "Bird Tweet",  # 124
    "Telephone Ring",  # 125
    "Helicopter",  # 126
    "Applause",  # 127
    "Gunshot",  # 128
]

assert len(GM1_INSTRUMENT_NAMES) == 128


def gm1_instrument_name(program):
    """ GM 1 instrument name. See also `gm1_instrument_familly_name`. """
    index = program
    if index >= len(GM1_INSTRUMENT_NAMES):
        raise RuntimeError
    return GM1_INSTRUMENT_NAMES[index]


# GM1 Instrument Familly Names
# ============================================================================

# For use with `midi.ProgramChange` events.
# To get the familly index, divide the program number by 8 (integer division).
# Note that display number start at 1, while its integer code start at 0.
# See also `GM_INSTRUMENT_NAMES`.
# Indexed by zero based program number divided by 8.
GM1_INSTRUMENT_FAMILLY_NAMES = [
    "Piano",  # 1-8
    "Chromatic Percussion",  # 9-16
    "Organ",  # 17-24
    "Guitar",  # 25-32
    "Bass",  # 33-40
    "Strings",  # 41-48
    "Ensemble",  # 49-56
    "Brass",  # 57-64
    "Reed",   # 65-72
    "Pipe",  # 73-80
    "Synth Lead",  # 81-88
    "Synth Pad",  # 89-96
    "Synth Effects",  # 97-104
    "Ethnic",  # 105-112
    "Percussive",  # 113-120
    "Sound Effects",  # 121-128
]

assert len(GM1_INSTRUMENT_NAMES) % 8 == 0
assert len(GM1_INSTRUMENT_FAMILLY_NAMES) == len(GM1_INSTRUMENT_NAMES) // 8


def gm1_instrument_familly_name(program):
    """ GM 1 instrument familly name. See also `gm1_instrument_name`. """
    index = program // 8
    if index >= len(GM1_INSTRUMENT_FAMILLY_NAMES):
        raise RuntimeError
    return GM1_INSTRUMENT_FAMILLY_NAMES[index]


# GM1 Percussion Names
# ============================================================================

# For use with `channel_event.Note` events.
# Key numbers start at 0, unlike channel and instrument numbers.
# Indexed by zero based key number.

FIRST_PERCUSSION_INDEX = 35
LAST_PERCUSSION_INDEX = 81
N_PERCUSSIONS = LAST_PERCUSSION_INDEX - FIRST_PERCUSSION_INDEX + 1
assert N_PERCUSSIONS == 47

GM1_PERCUSSION_NAMES = [
    "Acoustic Bass Drum",  # 35
    "Bass Drum 1",  # 36
    "Side Stick",  # 37
    "Acoustic Snare",  # 38
    "Hand Clap",  # 39
    "Electric Snare",  # 40
    "Low Floor Tom",  # 41
    "Closed Hi Hat",  # 42
    "High Floor Tom",  # 43
    "Pedal Hi-Hat",  # 44
    "Low Tom",  # 45
    "Open Hi-Hat",  # 46
    "Low-Mid Tom",  # 47
    "Hi-Mid Tom",  # 48
    "Crash Cymbal 1",  # 49
    "High Tom",  # 50
    "Ride Cymbal 1",  # 51
    "Chinese Cymbal",  # 52
    "Ride Bell",  # 53
    "Tambourine",  # 54
    "Splash Cymbal",  # 55
    "Cowbell",  # 56
    "Crash Cymbal 2",  # 57
    "Vibraslap",  # 58
    "Ride Cymbal 2",  # 59
    "Hi Bongo",  # 60
    "Low Bongo",  # 61
    "Mute Hi Conga",  # 62
    "Open Hi Conga",  # 63
    "Low Conga",  # 64
    "High Timbale",  # 65
    "Low Timbale",  # 66
    "High Agogo",  # 67
    "Low Agogo",  # 68
    "Cabasa",  # 69
    "Maracas",  # 70
    "Short Whistle",  # 71
    "Long Whistle",  # 72
    "Short Guiro",  # 73
    "Long Guiro",  # 74
    "Claves",  # 75
    "Hi Wood Block",  # 76
    "Low Wood Block",  # 77
    "Mute Cuica",  # 78
    "Open Cuica",  # 79
    "Mute Triangle",  # 80
    "Open Triangle",  # 81
]

assert len(GM1_PERCUSSION_NAMES) == N_PERCUSSIONS

UNKNOWN_PERCUSSION = "Unknown Percussion"


def gm1_percussion_name(key):
    """ GM 1 percussion name. """
    if key >= FIRST_PERCUSSION_INDEX and key <= LAST_PERCUSSION_INDEX:
        result = GM1_PERCUSSION_NAMES[key - FIRST_PERCUSSION_INDEX]
    else:
        result = UNKNOWN_PERCUSSION
    return result
