# -*- coding:utf-8; mode:python3; indent-tabs-mode:nil; c-basic-offset:4; -*-

""" Manufacturer names and region names from their index. """

from . import manufacturers

from .manufacturers import Indexes as Manufacturers


# Names
# ============================================================================

NAMES = {

    # One byte IDs
    Manufacturers.SEQUENTIAL_CIRCUITS: "Sequential Circuits",
    Manufacturers.BIG_BRIAR: "Big Briar",
    Manufacturers.OCTAVE_PLATEAU: "Octave / Plateau",
    Manufacturers.MOOG: "Moog",
    Manufacturers.PASSPORT_DESIGNS: "Passport Designs",
    Manufacturers.LEXICON: "Lexicon",
    Manufacturers.KURZWEIL: "Kurzweil",
    Manufacturers.FENDER: "Fender",
    Manufacturers.GULBRANSEN: "Gulbransen",
    Manufacturers.DELTA_LABS: "Delta Labs",
    Manufacturers.SOUND_COMP: "Sound Comp.",
    Manufacturers.GENERAL_ELECTRO: "General Electro",
    Manufacturers.TECHMAR: "Techmar",
    Manufacturers.MATTHEWS_RESEARCH: "Matthews Research",
    Manufacturers.OBERHEIM: "Oberheim",
    Manufacturers.PAIA: "PAIA",
    Manufacturers.SIMMONS: "Simmons",
    Manufacturers.DIGIDESIGN: "DigiDesign",
    Manufacturers.FAIRLIGHT: "Fairlight",
    Manufacturers.PEAVEY: "Peavey",
    Manufacturers.JL_COOPER: "JL Cooper",
    Manufacturers.LOWERY: "Lowery",
    Manufacturers.LIN: "Lin",
    Manufacturers.EMU: "Emu",
    Manufacturers.BON_TEMPI: "Bon Tempi",
    Manufacturers.SIEL: "S.I.E.L.",
    Manufacturers.SYNTHEAXE: "SyntheAxe",
    Manufacturers.HOHNER: "Hohner",
    Manufacturers.CRUMAR: "Crumar",
    Manufacturers.SOLTON: "Solton",
    Manufacturers.JELLINGHAUS_MS: "Jellinghaus Ms",
    Manufacturers.CTS: "CTS",
    Manufacturers.PPG: "PPG",
    Manufacturers.ELKA: "Elka",
    Manufacturers.CHEETAH: "Cheetah",
    Manufacturers.WALDORF: "Waldorf",
    Manufacturers.KAWAI: "Kawai",
    Manufacturers.ROLAND: "Roland",
    Manufacturers.KORG: "Korg",
    Manufacturers.YAMAHA: "Yamaha",
    Manufacturers.CASIO: "Casio",
    Manufacturers.AKAI: "Akai",
    Manufacturers.EDUCATIONAL: "Educational",
    Manufacturers.NON_REAL_TIME_SYSEX: "Non‑Real Time SysEx",
    Manufacturers.REAL_TIME_SYSEX: "Real Time SysEx",

    # Three bytes IDs
    Manufacturers.MICROSOFT: "Microsoft",
}

assert len(NAMES) == len(Manufacturers)
assert all(x.value in NAMES for x in Manufacturers)


UNKNOWN = "Unknown Manufacturer"


def name(index):
    """ Manufacturer name from ID. """
    if index in NAMES:
        result = NAMES[index]
    else:
        result = UNKNOWN
    return result


# Regions
# ============================================================================

# http://search.cpan.org
# /~hayashi/Win32API-MIDI-0.05/MIDI/SysEX.pm#Manufacturer%27s_ID_Number

AMERICAN_MANUFACTURER = "American Manufacturer"
EUROPEAN_MANUFACTURER = "European Manufacturer"
JAPANESE_MANUFACTURER = "Japanese Manufacturer"
OTHER_MANUFACTURER = "Other Manufacturer"
UNKNOWN_REGION = "Unknown Region"


# American Region
# ----------------------------------------------------------------------------

AMERICAN_1_BYTE_FIRST = manufacturers.index(bytes([0x01]))
AMERICAN_1_BYTE_LAST = manufacturers.index(bytes([0x1F]))
AMERICAN_2_BYTES_FIRST = manufacturers.index(bytes([0x00, 0x00, 0x01]))
AMERICAN_2_BYTES_LAST = manufacturers.index(bytes([0x00, 0x1F, 0x7F]))


def is_american(index):
    """ Is in one of the two American ranges. """
    if index >= AMERICAN_1_BYTE_FIRST and index <= AMERICAN_1_BYTE_LAST:
        result = True
    elif index >= AMERICAN_2_BYTES_FIRST and index <= AMERICAN_2_BYTES_LAST:
        result = True
    else:
        result = False
    return result


# European Region
# ----------------------------------------------------------------------------

EUROPEAN_1_BYTE_FIRST = manufacturers.index(bytes([0x20]))
EUROPEAN_1_BYTE_LAST = manufacturers.index(bytes([0x3F]))
EUROPEAN_2_BYTES_FIRST = manufacturers.index(bytes([0x00, 0x20, 0x00]))
EUROPEAN_2_BYTES_LAST = manufacturers.index(bytes([0x00, 0x3F, 0x7F]))


def is_european(index):
    """ Is in one of the two European ranges. """
    if index >= EUROPEAN_1_BYTE_FIRST and index <= EUROPEAN_1_BYTE_LAST:
        result = True
    elif index >= EUROPEAN_2_BYTES_FIRST and index <= EUROPEAN_2_BYTES_LAST:
        result = True
    else:
        result = False
    return result


# Japanese Region
# ----------------------------------------------------------------------------

JAPANESE_1_BYTE_FIRST = manufacturers.index(bytes([0x40]))
JAPANESE_1_BYTE_LAST = manufacturers.index(bytes([0x5F]))
JAPANESE_2_BYTES_FIRST = manufacturers.index(bytes([0x00, 0x40, 0x00]))
JAPANESE_2_BYTES_LAST = manufacturers.index(bytes([0x00, 0x5F, 0x7F]))


def is_japanese(index):
    """ Is in one of the two Japanese ranges. """
    if index >= JAPANESE_1_BYTE_FIRST and index <= JAPANESE_1_BYTE_LAST:
        result = True
    elif index >= JAPANESE_2_BYTES_FIRST and index <= JAPANESE_2_BYTES_LAST:
        result = True
    else:
        result = False
    return result


# Other Region
# ----------------------------------------------------------------------------

OTHER_1_BYTE_FIRST = manufacturers.index(bytes([0x60]))
OTHER_1_BYTE_LAST = manufacturers.index(bytes([0x7C]))
OTHER_2_BYTES_FIRST = manufacturers.index(bytes([0x00, 0x60, 0x00]))
OTHER_2_BYTES_LAST = manufacturers.index(bytes([0x00, 0x7F, 0x7F]))


def is_other(index):
    """ Is in one of the two Other ranges. """
    if index >= OTHER_1_BYTE_FIRST and index <= OTHER_1_BYTE_LAST:
        result = True
    elif index >= OTHER_2_BYTES_FIRST and index <= OTHER_2_BYTES_LAST:
        result = True
    else:
        result = False
    return result


# Main
# ----------------------------------------------------------------------------

def region(index):
    """ Manufacturer region of the world, from ID. """
    if is_american(index):
        result = AMERICAN_MANUFACTURER
    elif is_european(index):
        result = EUROPEAN_MANUFACTURER
    elif is_japanese(index):
        result = JAPANESE_MANUFACTURER
    elif is_other(index):
        result = OTHER_MANUFACTURER
    else:
        result = UNKNOWN_REGION
    return result
