# -*- coding:utf-8; mode:python3; indent-tabs-mode:nil; c-basic-offset:4; -*-

""" Manufacturer IDs. """

from collections import namedtuple
from enum import IntEnum

from . import midi


ManufacturerData = namedtuple("ManufacturerData", ["manufacturer", "data"])


def index(id_data):
    """ ID to index. """
    if len(id_data) == 1:
        result = id_data[0]
        if result == 0:
            raise RuntimeError
    elif len(id_data) == 3:
        if id_data[0] != 0:
            raise ValueError
        result = midi.be14bits(id_data[1], id_data[2])
        if result == 0:
            raise ValueError
        result = 0x80 + result
    else:
        raise RuntimeError
    return result


def decoded_data(event_data):
    """ `ManufacturerData` from event data. """
    if len(event_data) < 1:
        raise ValueError
    if event_data[0] != 0:
        manufacturer = index(event_data[0:1])
        data = event_data[1:]
    else:
        if len(event_data) < 3:
            raise ValueError
        # Three bytes ID as 14 bits stored on 15 bits with bit #14 set.
        manufacturer = index(event_data[0:3])
        data = event_data[3:]
    return ManufacturerData(manufacturer=manufacturer, data=data)


class Indexes(IntEnum):

    """ Manufacturer IDs. """

    # One byte IDs
    SEQUENTIAL_CIRCUITS = index(bytes([0x01]))
    BIG_BRIAR = index(bytes([0x02]))
    OCTAVE_PLATEAU = index(bytes([0x03]))
    MOOG = index(bytes([0x04]))
    PASSPORT_DESIGNS = index(bytes([0x05]))
    LEXICON = index(bytes([0x06]))
    KURZWEIL = index(bytes([0x07]))
    FENDER = index(bytes([0x08]))
    GULBRANSEN = index(bytes([0x09]))
    DELTA_LABS = index(bytes([0x0A]))
    SOUND_COMP = index(bytes([0x0B]))
    GENERAL_ELECTRO = index(bytes([0x0C]))
    TECHMAR = index(bytes([0x0D]))
    MATTHEWS_RESEARCH = index(bytes([0x0E]))
    OBERHEIM = index(bytes([0x10]))
    PAIA = index(bytes([0x11]))
    SIMMONS = index(bytes([0x12]))
    DIGIDESIGN = index(bytes([0x13]))
    FAIRLIGHT = index(bytes([0x14]))
    PEAVEY = index(bytes([0x1B]))
    JL_COOPER = index(bytes([0x15]))
    LOWERY = index(bytes([0x16]))
    LIN = index(bytes([0x17]))
    EMU = index(bytes([0x18]))
    BON_TEMPI = index(bytes([0x20]))
    SIEL = index(bytes([0x21]))
    SYNTHEAXE = index(bytes([0x23]))
    HOHNER = index(bytes([0x24]))
    CRUMAR = index(bytes([0x25]))
    SOLTON = index(bytes([0x26]))
    JELLINGHAUS_MS = index(bytes([0x27]))
    CTS = index(bytes([0x28]))
    PPG = index(bytes([0x29]))
    ELKA = index(bytes([0x2F]))
    CHEETAH = index(bytes([0x36]))
    WALDORF = index(bytes([0x3E]))
    KAWAI = index(bytes([0x40]))
    ROLAND = index(bytes([0x41]))
    KORG = index(bytes([0x42]))
    YAMAHA = index(bytes([0x43]))
    CASIO = index(bytes([0x44]))
    AKAI = index(bytes([0x45]))
    EDUCATIONAL = index(bytes([0x7D]))
    NON_REAL_TIME_SYSEX = index(bytes([0x7E]))
    REAL_TIME_SYSEX = index(bytes([0x7F]))

    # Three bytes IDs
    MICROSOFT = index(bytes([0x0, 0x00, 0x41]))


assert len({x.value for x in Indexes}) == len(Indexes)
